
/*
algo-lab-source.js
class: AlgoLabSource

The source code for the algorithm lab.

Contains:
    source -- the source code
    selection
        -- (pos1, pos2) that models where the current selection
        is in the source. If pos1 == pos2, then no selection, or
        if either is null
*/

class AlgoLabSource {

    constructor() {
    }

    //-------------------------------------------------------------------
    // Accessors

    source(arg) {
        if (arg !== undefined) {
            console.log('SETTING SOURCE');
            console.log(arg);
            console.log('');
            this._source = arg;
        }
        if (this._source == null)
            this._source = '';
        return this._source;
    }

    selectionRange(arg) {
        if (arg !== undefined) this.setSelectionRange(arg);
        if (this._selectionRange === undefined)
            this._selectionRange = this.selectionRangeBlt();
        return this._selectionRange;
    }

    hasSelection() {
        const seln = this._selectionRange;
        return seln && ((seln[1] - seln[0]) > 0);
    }

    selectionLength() {
        const seln = this._selectionRange;
        if (!seln) return 0;
        return seln[1] - seln[0];
    }

    cursorPos() {
        //return first coord of range
        return this.selectionRange()[0];
    }

    setCursorPos(pos) {
        //set both coords of range to pos
        this.selectionRange([pos, pos]);
    }

    currentCharacter() {
        return this.source().charAt(this.cursorPos());
    }

    selectionRangeBlt() {
        //Special behavior check if session has cached value
        //try w/o cache
        /*
        const string = sessionStorage.getItem('selectionRange');
        if (string !== undefined) {
            sessionStorage.removeItem('selectionRange');    //clear cached value
            const pair = string.split('|');
            return [parseInt(pair[0]), parseInt(pair[1])];
        }
        */
        //default
        return [0, 0];
    }

    setSelectionRange(arg) {
        //Special behavior cache into session so we don't lose on refresh
        //console.log('setting selection range: ' + arg);
        console.log('SETTING SELECTION RANGE');
        console.log(arg);

        if (!Array.isArray(arg))
            throw `please send pair to "setSelectionRange" -- received (${arg})`;

        this._selectionRange = arg;
        //try w/o cache
        /*if (arg)
            sessionStorage.setItem("selectionRange", arg.join('|'));*/
    }

    getSelectedText() {
        //return selected text (may be empty string)
        const
            text = this.source(),
            pair = this.selectionRange(),
            i1 = (pair ? pair[0] : 0),
            i2 = (pair ? pair[1] : 0);
        if (i1 != null && i2 != null)
            return text.slice(i1, i2);
        else
            return "";
    }

    asStream() {
        // eslint-disable-next-line no-undef
        let strm = new TextStream(this.source());
        strm.setPos(this.cursorPos());
        return strm;
    }

    errorRangeFor(error) {
        let src, startPos, errorLineText, len, searchTarget, pattern, match, escapedLine;
        src = this.source();
        startPos = this.constructor.getErrorSearchStartPos(error, src);
        errorLineText = error.getLineText();
        if (!errorLineText) return null;
        len = errorLineText.length;
        searchTarget = src.slice(startPos) + '\n';  //better regex behavior with trailing '\n'

        escapedLine = RegExp.escape(errorLineText);
        pattern = new RegExp(`^\\s*${escapedLine}\\s*\\n`);
        const i = searchTarget.search(pattern);
        if (i === 0)
            return [startPos, startPos + len];
        match = searchTarget.match(pattern);
        if (!match)
            return null;
        //find offset match (beyond whitespace)
        const matchIndex = match.index;
        const src2 = searchTarget.slice(matchIndex);
        pattern = new RegExp(`\\n\\s*${escapedLine}\\s*\\n`);
        const offset = src2.search(pattern);
        if (offset < 0)
            throw 'regular expression error match error';
        const matchOffset = matchIndex + offset;
        return [startPos + matchOffset, startPos + matchOffset + len];
    }

    insert(newText) {
        let p1, s;
        //insert after cp
        p1 = this.cursorPos() + 1;
        s = this.source();
        this.source(s.slice(0, p1) + newText + s.slice(p1));
    }

    getVarTypeMap() {
        // eslint-disable-next-line no-undef
        return (new PcodeParser(this.source())).varTypeMap();
    }

    typeAtDotOperator() {
        let strm, varName;
        strm = this.asStream();
        if (strm.peek() !== '.')
            return null;
        strm.back();
        varName = strm.backToFor(ch => !isValidVarNameChar(ch));
        return this.getVarTypeMap().get(varName);
    }

    //------------------------------------------------------------
    // Static Helpers

    static getErrorSearchStartPos(error, src) {
        let lineNo, strm;
        lineNo = error.getLineNumber();
        if (!lineNo) return 0;
        // eslint-disable-next-line no-undef
        strm = TextStream.from(src);
        strm.repeat(aStrm => aStrm.nextLine(), lineNo - 1);
        return strm.pos();
    }

}

//-------------------------------------------------------------
AlgoLabSource;