
class AlgoMethodSet {

    /*ivars and derived vars
        zclass
        methods
        headers
        instanceOption
    */

    constructor(aClass, anInstanceOption = true) {
        this.setZclass(aClass);
        this.instanceOption(anInstanceOption);
    }

    //-------------------------------------------------------------------
    // accessing

    instanceOption(arg) {
        if (arg !== undefined) {
            this._instanceOption = arg;
            this.clearDerived();
        }
        return this._instanceOption;
    }

    getZclass() {
        return this._zclass;
    }

    setZclass(cl) {
        this._zclass = cl;
    }

    //-------------------------------------------------------------------
    // methods

    methods(arg) {
        if (arg !== undefined)
            this._methods = arg;
        else if (this._methods == null) this._methods = this.methodsBlt();
        return this._methods;
    }

    methodsBlt() {
        const cl = this.getZclass();
        //cl = zclass
        if (!cl) return [];
        //source is a class or metaclass
        const instanceOption = this.instanceOption();
        const source = instanceOption ? cl.prototype : cl;
        const methodNames = Object.getOwnPropertyNames(source);
        let result = methodNames.map(nm => [nm, source[nm]]);
        //Let's not show the constructor, we'll force
        //the use of friendlier static factory methods
        if (instanceOption)
            this.constructor.removeConstructor(result, cl);
        else {
            result = result.filter(pair => typeof pair[1] === 'function');
            //this.constructor.addConstructor(result, cl);
        }
        //TRY -- try w/o sort so they stay somewhat in "groups"
        //return result.sort((a, b) => a[0].compareTo(b[0]));
        return this.constructor.rejectPrivate(result);
    }

    static rejectPrivate(methodPairs) {
        //pair[0] is method name, pair[1] is method function object
        return methodPairs.filter(pair => !pair[1].toString().includes('//private'));
    }

    static removeConstructor(methods, cl) {
        const index = methods.findIndex(pair => {
            return pair[1].name == cl.name
                || pair[0] == 'constructor';
        });
        if (index >= 0)
            methods.splice(index, 1);
    }

    static addConstructor(methods, cl) {
        const index = methods.findIndex(ea => ea.name === cl.name);
        if (index >= 0) return;
        const constructor = cl.prototype['constructor'];
        if (!constructor) return;
        methods.add(constructor);
    }

    //-------------------------------------------------------------------
    // method headers

    methodHeaders(arg) {
        if (arg !== undefined) this._methodHeaders = arg;
        else if (this._methodHeaders == null) this._methodHeaders = this.methodHeadersBlt();
        return this._methodHeaders;
    }

    methodHeadersBlt() {
        return this.methods().map(pair => this.constructor.methodHeaderFrom(pair, this.getZclass().name, !this.instanceOption()));
    }

    //-------------------------------------------------------------------
    //private -- deriving method header

    static methodHeaderFrom(pair, className, boolStatic) {
        //Answer method name and arg names
        //foo(w, h)
        const nm = pair[0];
        const method = pair[1];
        if (nm === className || nm === 'constructor')
            return this.constructorMethodHeaderFrom(method, className);
        const prefix = boolStatic ? 'static ' : '';
        const header = method.name.trim().length > 0
            ? this.basicMethodHeaderFrom(nm, method, className)
            : this.basicMethodHeaderFromPrototype(nm, method, className);
        return prefix + header;
    }

    static basicMethodHeaderFrom(nm, method, className) {
        const right = ")";
        return method.toString().split(right)[0] + right;
    }

    static basicMethodHeaderFromPrototype(nm, method, className) {
        //Answer method name and arg names
        //foo(w, h)
        let ss = method.toString();
        if (!(ss.includes('(') && ss.includes(')')))
            return nm;
        ss = method.toString().split(')')[0];
        ss = ss.split('(')[1];
        return `${nm}(${ss})`;
    }

    static constructorMethodHeaderFrom(method, className) {
        //match any white + [constructor] + [(] + any chars + [{]
        //return in format "MyArray(10)"
        let header = method.toString().match("\\s*constructor[(].*[{]");
        if (header == null)
            return null;
        header = header[0].trim();
        if (header.endsWith('{'))
            header = header.slice(0, -1);
        header = header.slice('constructor'.length);
        return className + header;
    }

    //-------------------------------------------------------------------
    //private -- deriving method header

    clearDerived() {
        this.methods(null);
        this.methodHeaders(null);
    }

    //-------------------------------------------------------------------
    //public services

    findMethodNamed(aMethodName) {
        const nm = (this._zclass.name === aMethodName) ? 'constructor' : aMethodName;
        return this.methods().find(pair => pair[0] == nm);
    }

}

//-------------------------------------------------------------
AlgoMethodSet;