
/*
source-error.js
class: SourceError

I have an error (text) and line number of error (which may be -1 if unknown)
Line number is 1-based

Line text is optional, but may be for case, e.g. where
we capture full line text where error occurred
*/

class SourceError {

    constructor(message, lineText, lineNumber) {
        this.message = message;
        this.lineText = lineText;
        this.lineNumber = lineNumber;
    }

    static fromMessageLine(msg, lineText, lineNo) {
        return new this(msg, lineText, lineNo);
    }

    static fromMessage(msg) {
        return new this(msg, null, null);
    }

    static newError() {
        return new this(null, null, null);
    }

    //-------------------------------------------------------------

    clear() {
        this.lineNumber = -1;
        this.errorMessage = null;
        this.lineText = null;
    }

    hasError() {
        return this.getMessage() != null && this.getMessage().length > 0;
    }

    hasLineNumber() {
        return this.getLineNumber() != null && this.getLineNumber() >= 1;
    }

    getMessage() {
        return this.message;
    }

    getLineText() {
        return this.lineText;
    }

    getLineNumber() {
        return this.lineNumber;
    }


    setMessage(msg) {
        this.message = msg;
    }

    setLineText(text) {
        this.lineText = text;
    }

    setLineNumber(no) {
        this.lineNumber = no;
    }

}

//-------------------------------------------------------------
SourceError;