
//note: next line is for validation system
/*global addTestCase, TestCase, AlgoLabModel, SourceValidator*/

//framework - argument here should be name of this test case
addTestCase("SourceValidatorTest");

class SourceValidatorTest extends TestCase {

	//-----------------------------
	//Constructors

	//-----------------------------
	//Helpers

	getLogSwitchFct() {
		return AlgoLabModel.getLogSwitchFct();
	}

	//-----------------------------
	//Test Methods

	//------------

	test_ReferenceError_1() {
		let src, validator, error;
		src =
`log('hey')
log('ho')
badCallHere()
log('wow)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(3, error.getLineNumber());
	}

	//------------

	test_Unexpected_identifier_1() {
		let src, validator, error;
		src =
`let j = 3
letoops i = 0
let foo = 3`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(2, error.getLineNumber());
	}

	test_Unexpected_identifier_2() {
		let src, validator, error;
		src =
`let a = 10
let b = 20
log('a: ' + a)
log('b: ' + b)
a b
log(sum)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(5, error.getLineNumber());
	}


	//------------

	test_Unexpected_token_1() {
		let src, validator, error;
		src =
`

let i = 1
whilee (i < 5) {
	log(i)
	i++
}`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(4, error.getLineNumber());
	}

	//------------

	test_extraBrace_1() {
		let src, validator, error;
		src =
`log('hey')
log('ho') }
log('wow)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(2, error.getLineNumber());
	}

	test_unexpectedEndOfInput_1() {
		let src, validator, error;
		src =
`log('hey')
log('ho') *
log('wow)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(2, error.getLineNumber());
	}

	test_unexpectedEndOfInput_2() {
		let src, validator, error;
		src =
`let i = 1
while (i < 5) {
	log(i)
	i++`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(2, error.getLineNumber());
	}

	//------------

	test_isNotDefined_1() {
		let src, validator, error;
		src =
`let a = 10
let b = 20
log('a: ' + a)
log('b: ' + b)
sum = a + b
log(sum)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(5, error.getLineNumber());
		this.assertTrue(error.getMessage().includes('sum'));
	}


	test_isNotDefined_2() {
		let src, validator, error;
		src =
`let a = 10
let b = 20
let sum = aoops + b
log(sum)`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(3, error.getLineNumber());
		this.assertTrue(error.getMessage().includes('aoops'));
	}

	//------------

	test_misc_1() {
		let src, validator, error;
		src =
`let a = 10
let b = 20
log('a: ' + a)
log('b: ' + b)
let sum = a + b
log(sum`;
		validator = SourceValidator.fromSource(src, this.getLogSwitchFct());
		validator.validate();
		error = validator.getError();
		this.assertEquals(6, error.getLineNumber());
		this.assertTrue(error.getMessage().includes('missing') && error.getMessage().includes(')'));
	}


}

//-----------------------------------------------------

// if (require.main === module)
// 	MyStringTest.runAll();

SourceValidatorTest;
