/*
MyArray - is an enhanced array object.
It provides some nice features such
as "indexOf, reversed, map, join, includes"
as being able to split, reformat, capitlaize words and more

SPECIAL NOTES:

	Return value:
		If the return type is [not] noted in the method
		comment, then assume it is an array.
		Note that var "index" is an integer type
*/

class MyArray {

	//---------------------------------
	// Services

	//==================================================================================
	//==================================================================================
	//==================================================================================
	//==================================================================================
	// Provided Logic (Finished)

	//---------------------------------
	//Constructors

	constructor(aElements) {
		/*
		Constructor
		Usage:
		let myArray = new MyArray([10, 20, 30, 40])
		*/
		this.privSetElements(aElements);
	}

	static from(elements) {
		/*Static factory constructor
		e.g.,
		let a = MyArray.from([10, 20, 30, 40])*/
		return new MyArray(elements);
	}

	static fromSize(sz) {
		/*Return a new array with elem count of "sz"
		Elements are all initialized to null
		e.g.,
		let a = MyArray.fromSize(10)*/
		let fixedArray = new Array(sz);
		fixedArray.fill(null);
		return this.from(fixedArray);
	}

	//---------------------------------
	//Accessors

	privGetElements() {
		//private -- Return my elements (plain array)
		return this._elements;
	}

	privSetElements(plainArray) {
		//private -- Set my elements with passed arg (plain array)
		this._elements = plainArray;
	}

	//---------------------------------
	//Helpers

	size() {
		/* Return my size (length)
		In other words the number of elements
		e.g.,
		let array = MyArray.from([10, 20, 30, 40])
		println('size: ' + array.size())
		//size: 4*/
		return this.privGetElements().length;
	}

	toString() {
		/* Returns a "nice string" (one-line) about this object
		e.g.,
		let array = MyArray.from([10, 20, 30, 40])
		println(array.toString())
		//elements: [10, 20, 30, 40]*/
		return 'elements: ' + this.privGetElements().toString();
	}

	get(index) {
		/* I return my array element at the index passed in
		Indexes are zero-based 0, 1, 2, ...
		e.g.,
		let array = MyArray.from([10, 20, 30, 40])
		println('get(0): ' + array.get(0))
		//get(0): 10
		println('get(2): ' + array.get(2))
		//get(0): 30*/
		return this.privGetElements()[index];
	}

	set(index, newElem) {
		/* Sets (replaces) the element at "index"
		with "newElem"
		e.g.,
		let array = MyArray.from([10, 20, 30, 40])
		array.set(1, 101)
		println(array.toString()))
		//elements: [10, 101, 30, 40]
		array.set(3, 999)
		println(array.toString()))
		//elements: [10, 101, 30, 999]*/
		return this.privGetElements()[index] = newElem;
	}

	equals(anArray) {
		/* Returns true if this array equals arg "anArray".
		Arg "anArray" is a "MyArray"
		e.g.,
		let a1 = MyArray.from([10, 20, 30])
		let a2 = MyArray.from([10, 20, 30])
		let a3 = MyArray.from([99, 20, 30])
		println('Compare a1 and a2: ' + a1.equals(a2))
		//Compare a1 and a2: true
		println('Compare a1 and a3: ' + a1.equals(a3))
		//Compare a1 and a3: false
		*/
		let otherArray =
			(anArray.constructor.name === this.constructor.name)
				? anArray.privGetElements()
				: anArray;
		return this.equalsArray(otherArray);
	}

	//---------------------------------
	// Helpers (on all elements)

	printAll(label) {
		if (label)
			println('-- ' + label);
		if (this.size() == 0) {
			println('(empty array)');
			return;
		}
		if (this.get(0).headers)
			println(this.get(0).headers());
		this.allDo(eachElem => println(eachElem.toString()));
		println('');
	}

	allDo(fct) {
		for (let eachElem of this.privGetElements())
			fct(eachElem);
	}


	//---------------------------------
	// Private

	equalsArray(array) {
		//private
		if (!Array.isArray(array))
			return false;
		let size = this.size();
		if (this.size() !== array.length)
			return false;
		//okay, loop thru elems
		for (let i = 0; i < size; i++)
			if (this.get(i) !== array[i])
				return false;
		//all checks passed, so return true
		return true;
	}

}

//---------------------------------------------------

//exports.MyArray = MyArray;
//Just for reference
MyArray;